import { MQTT } from './paho-mqtt'

export = xclient
declare function xclient(options: xclient.ClientOptions): xclient.XClient

declare namespace xclient {
  export type LogLanguage = 'zh-CN' | 'en-US'

  export type LogType = 'success' | 'error'

  export interface Log {
    type: LogType;
    message: string;
    error?: MQTT.ErrorWithInvocationContext
  }

  export interface Payload {
    [key: string]: any
  }

  export interface ClientOptions {
    host: string; // 服务器
    clientId: string; // 客户端 ID
    userName: string; // 用户名
    password: string; // 密码
    mqttVersion?: 4 | 3; // MQTT 版本
    keepAliveInterval?: number; // 心跳包间隔
    lang?: LogLanguage // 日志语言
    debug?: boolean // 是否进入 debug 模式，该模式下会进行所有的信息打印
    onConnectionLost?: Function // 连接丢失的回调
    onMessageArrived?: (message: { destinationName: string; payload: Payload }) => void // 接收到信息的回调
  }

  class XClient {
    constructor(option: ClientOptions)

    connect(): Promise<MQTT.WithInvocationContext>

    disconnect(): void

    subscribe(topic: string): Promise<MQTT.OnSubscribeSuccessParams>

    unsubscribe(topic: string): Promise<MQTT.WithInvocationContext>

    send(topic: string, payload: ArrayBuffer): void

    isConnected(): boolean
  }

  export type PluginFunc = (options: any, baseClass: typeof XClient, instance: typeof xclient) => void

  export function extend(plugin: PluginFunc, options?: any): XClient

  export const topics: {
    [key: string]: string
  }

}
